var async = require('async');
var moment = require('moment');
var fs = require('fs-extra');

var defaultDir = '/usr/src/app/userdata/networks';
var homeDir = '/home/seluser/';

function screenshot (client) {
    var fileName = moment().format('YYYYMMDDHHmmss');
    var screenShotPath = './automated-tests/screenshots/';
    var fileFormat = '.png';
    var filePath = screenShotPath+fileName+fileFormat;
    
    client.saveScreenshot(filePath);
}

module.exports = {
    '@tags': ['fm_dirBrowser'],

    before: function (client) {
        client
            .url('http://localhost:3000')
            .waitForElementVisible('#topmenu', 1000);
    },

    after: function (client) {
        // Removes fixtures
        var path = homeDir + 'Test';
        fs.removeSync(path);

        client.end();
    },

    /**
     *  Test: 2-001
     *  Expected:
     *      - Display file explorer/directory browser dialog
     *      - Display the networks directory
     * */
    "2-001": function (client) {
        var fileManagement = client.page.fileManagement();

        // Opens file manager
        fileManagement.open();

        // Opens the directory browser
        fileManagement.openDirBrowser();

        // Asserts directory browser dialog to display
        fileManagement.expect.element('@fileTree').to.be.visible;

        // Asserts the directory is listed
        fileManagement.expect.element('@fileTree').to.have.attribute('aria-busy').which.contains('false').after(1000);
    },

    /**
     *  Test: 2-002
     *  Expected:
     *      - File explorer dialog should be closed
     * */
    "2-002": function (client) {
        var fileManagement = client.page.fileManagement();

        // Clicks close button
        fileManagement.closeDirBrowser();

        // Asserts directory browser to not visible
        fileManagement.expect.element('@fileTreeDialog').to.not.be.visible;

        // Opens the directory browser
        fileManagement.openDirBrowser();

        // Clicks cancel button
        fileManagement.cancelDirBrowser();

        // Asserts directory browser to not visible
        fileManagement.expect.element('@fileTreeDialog').to.not.be.visible;
    },

    /**
     *  Test: 2-003
     *  Expected:
     *      - File list should display all the files in the Documents directory
     * */
    "2-003": function (client) {
        var fileManagement = client.page.fileManagement();
        var path = homeDir;

        async.series([
            function (callback) {
                // Clear value file path
                fileManagement.clearValue('@dataPath');

                // Set value file path
                fileManagement.setValue('@dataPath', path);

                callback();
            },
            function (callback) {
                // Press enter key
                client.keys(client.Keys.ENTER, function () {
                    client.pause(1000);
                    callback();
                });
            },
            function (callback) {
                // Assert file list to be visible
                fileManagement.expect.element('@fileList').to.be.visible;

                callback();
            }
        ], function (err, data) {
            // do nothing
        });

    },

    /**
     *  Test: 2-004
     *  Expected:
     *      - Directory browser dialog will be displayed
     *      - Documents directory and its subdirectories should be listed
     * */
    "2-004": function (client) {
        var fileManagement = client.page.fileManagement();

        // Open directory browser
        fileManagement.openDirBrowser();

        // Asserts directory browser to visible
        fileManagement.expect.element('@fileTreeDialog').to.be.visible;

        // Assert file tree to be visible
        fileManagement.expect.element('@fileTree').to.be.visible;

        // Assert that directory should be listed
        fileManagement.expect.element('@fileTree').to.have.attribute('aria-busy').which.contains('false').after(1000);

        // Close directory browser dialog
        fileManagement.cancelDirBrowser();
    },

    /**
     *  Test: 2-005
     *  Expected:
     *      - In the File explorer dialog, "Test" folder should be listed.
     * */
    "2-005": function (client) {
        var fileManagement = client.page.fileManagement();
        var path = homeDir + 'Test';

        // Make test directory
        if (!fs.existsSync(path)){
            fs.mkdirSync(path);
        }

        // Open directory browser
        fileManagement.openDirBrowser();

        // Assert directory to be visible
        fileManagement.expect.element('@fileTree').to.be.visible;
        fileManagement.expect.element('@fileTree').text.to.contain('Test').after(1000);
    },

    /**
     *  Test: 2-006
     *  Expected:
     *      - File explorer dialog should be closed
     *      - File list should display no files.
     *      - "No Records Found" label should be visible.
     *      - Path value should be updated to the directory of Test folder
     * */
    "2-006": function (client) {
        var fileManagement = client.page.fileManagement();
        var path = homeDir + 'Test';

        async.series([
            function (callback) {
                // Clicks test directory
                fileManagement.click('@testDir', function () {
                    client.pause(1000);
                    callback();
                });
            },
            function (callback) {
                // Selects the directory
                fileManagement.selectDir();

                // Asserts file tree dialog to be visible
                fileManagement.expect.element('@fileTreeDialog').not.be.visible;

                // Asserts file list to "No Records Found"
                fileManagement.expect.element('@fileList').text.to.contain("No Records Found");

                // Asserts file path equal to selected directory
                fileManagement.expect.element('@dataPath').to.have.value.that.equals(path);

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 2-007
     *  Expected:
     *      - File explorer dialog will be displayed
     *      - Test directory should be displayed in the list
     *      - A folder will be displayed under Test directory
     */
    "2-007": function (client) {
        var fileManagement = client.page.fileManagement();
        var path = homeDir + 'Test/A';

        // Makes the directory
        if (!fs.existsSync(path)){
            fs.mkdirSync(path);
        }

        async.series([
            function (callback) { // Test: 2-007
                // Open directory browser
                fileManagement.openDirBrowser();

                // Asserts directory browser to be visible
                fileManagement.expect.element('@fileTree').to.be.visible;

                // Asserts directory browser to have 'Test' directory
                fileManagement.expect.element('@fileTree').text.to.contain("Test").after(1000);

                // Asserts directory browser to have 'A' directory
                fileManagement.expect.element('@fileTree').text.to.contain("A").after(1000);

                callback();
            },
            function (callback) {
                // Clicks the directory
                fileManagement.click('@testDirSub', function () {
                    client.pause(1000);
                    callback();
                });
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 2-008
     *  Expected:
     *      - File explorer dialog should be closed
     *      - File list should display no files.
     *      - "No Records Found" label should be visible.
     *      - Path value should be updated to the directory of A folder
     * */
    "2-008": function (client) {
        var fileManagement = client.page.fileManagement();
        var path = homeDir + 'Test/A';

        async.series([
            function (callback) {
                // Selects the directory
                fileManagement.selectDir();

                // Test: 2-008
                // Assert if directory browser if not visible
                fileManagement.expect.element('@fileTreeDialog').not.be.visible;

                // Asserts file list to "No Records Found"
                fileManagement.expect.element('@fileList').text.to.contain("No Records Found");

                // Asserts file path equal to selected directory
                fileManagement.expect.element('@dataPath').to.have.value.that.equals(path);

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 2-009
     *  Expected:
     *      - File list will contain a.txt only.
     * */
    "2-009": function (client) {
        var fileManagement = client.page.fileManagement();
        var file = homeDir + 'Test/A/a.txt';

        // Creates text file
        fs.outputFileSync(file, "");

        // Clicks reload button
        fileManagement.clickReload();

        // Assert file list to contain 'a.txt'
        fileManagement.expect.element('@fileList').text.to.contain('a.txt').after(1000);
    },

    /**
     *  Test: 2-010
     *  Expected:
     *      - File list should display no files.
     *      - "No Records Found" label should be visible.
     * */
    "2-010": function (client) {
        var fileManagement = client.page.fileManagement();
        var path = homeDir + 'Test';

        async.series([
            function (callback) {
                // Sets path in file path
                fileManagement.clearValue('@dataPath');
                fileManagement.setValue('@dataPath', path);

                callback();
            },
            function (callback) {
                // Presses enter
                client.keys(client.Keys.ENTER, function () {
                    client.pause(1000);

                    callback();
                });
            },
            function (callback) {
                // Asserts file list to "No Records Found"
                fileManagement.expect.element('@fileList').text.to.contain("No Records Found");

                callback();
            }
        ], function (err, data) {
            // do nothing
        });
    },

    /**
     *  Test: 2-011
     *  Expected:
     *      - Path input should contain the default networks directory.
     *      - File list should contain all the files inside networks directory.
     * */
    "2-011": function (client) {
        var fileManagement = client.page.fileManagement();

        // Click home button
        fileManagement.clickHome();

        // Assert file path to contain default directory
        fileManagement.expect.element('@dataPath').value.to.equals(defaultDir);

        // Assert file list to contain files inside default directory
        fileManagement.expect.element('@fileList').text.to.not.equals('');
    },

    /**
     *  Test: 2-012
     *  Expected:
     *      - File list should display no files.
     *      - "No Records Found" label should be visible.
     * */
    "2-012": function (client) {
        var fileManagement = client.page.fileManagement();
        var path = '/invalidpath';
        var prevPath = '';
        var prevFilelist = '';

        async.series([
            function (callback) {
                // Gets the previous valid path
                fileManagement.getValue('@dataPath', function (result) {
                    prevPath = result.value;

                    callback();
                });
            },
            function (callback) {
                // Gets the previous valid path
                fileManagement.getText('@fileList', function (result) {
                    prevFilelist = result.value;

                    callback();
                });
            },
            function (callback) {
                // Sets path in file path
                fileManagement.clearValue('@dataPath');
                fileManagement.setValue('@dataPath', path);

                callback();
            },
            function (callback) {
                // Presses enter
                client.keys(client.Keys.ENTER, function () {
                    client.pause(1000);

                    callback();
                });
            },
            function (callback) {
                client.pause(1000, function () {
                    // Assert file list not to be updated
                    fileManagement.expect.element('@fileList').text.equals(prevFilelist);

                    // Assert file path to revert to previous valid path
                    fileManagement.expect.element('@dataPath').value.equals(prevPath);

                    // Assert to contain if error message is present
                    fileManagement.expect.element('@consoleStatus').text.to.contain("Error: Directory not found.");

                    callback();
                });

            }
        ], function (err, data) {
            // do nothing
        });
    }
};